#include <iostream>
#include <stdio.h>
#include "fisheyesw.h"
#include "fisheyesw_errordef.h"

#if defined ( _MSC_VER )
#include "SDL.h"
#pragma comment(lib, "SDL2main.lib")
#pragma comment(lib, "SDL2.lib")
#pragma comment(lib, "d_fisheyesw.lib")
#elif defined( __APPLE__ ) && defined( __MACH__ )
#include <SDL2/SDL.h>
#else
#include "SDL.h"
#include <string.h>
#endif

#ifdef main
#undef main
#endif

// keyboard/mouse event
void EventLoop(HANDLE hObject, FEOPTION *pOption, SDL_Texture* pTexture, SDL_Renderer *pRenderer);
SCODE UpdateOutVPicture(HANDLE hObject, FEOPTION *pOption, SDL_Texture* pTexture, SDL_Renderer *pRenderer);

#define IMAGE_NAME "sample.yuv"
#define MOUNT_TYPE 1

int main(int argc, char *argv[])
{
    const int WINDOW_WIDTH = 640, WINDOW_HEIGHT = 480;
    char *pszimgPath = NULL;
    int mountType = 0;
    const DWORD dwFovSize = 1920;
    FILE *pfImage = NULL;
    
    // SDL instance
    int sdlRet = 0;
    SDL_Texture *pTexture = NULL;
    SDL_Window *pScreen = NULL;
    SDL_Renderer *pRenderer = NULL;
    SDL_Surface* pBitmap = NULL;
    
    pszimgPath = IMAGE_NAME;
    mountType = MOUNT_TYPE;
    
    // SDL renderer
    sdlRet = SDL_Init(SDL_INIT_VIDEO);
    if (sdlRet)
    {
        return sdlRet;
    }
    if (NULL == (pScreen = SDL_CreateWindow("fisheyerect_yuv", SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED, WINDOW_WIDTH, WINDOW_HEIGHT, 0)))
    {
        return -1;
    }
    if (NULL == (pRenderer = SDL_CreateRenderer(pScreen, -1, 0)))
    {
        return -1;
    }
    if (NULL == (pTexture = SDL_CreateTexture(pRenderer, SDL_PIXELFORMAT_IYUV, SDL_TEXTUREACCESS_STREAMING, WINDOW_WIDTH, WINDOW_HEIGHT)))
    {
        return -1;
    }
    
    // Fisheye library instance
    SCODE scRet = FISHEYE_S_OK;
    HANDLE hFECtx = NULL;
	FEOPTION option;
	memset(&option, 0, sizeof(FEOPTION));

    // 1. Initialize fisheyesw library
    scRet = Fisheye_Initial(&hFECtx, LIBFISHEYE_VERSION);
    if (FISHEYE_S_OK != scRet)
    {
        goto RELEASE_MEMORY;
    }

    // Load image file
    if (NULL == (pfImage = fopen(pszimgPath, "rb")))
    {
        return -1;
    }
    
    // 2. Set fisheye options
    option.Scale = 100;  // default scale is 100 -> 1x zoom factor
    
    // Set input fisheye image information
    {
        // set input image information
        DWORD dwSrcBufferSize, dwDstBufferSize;
        BYTE *pbySrcBuffer = NULL, *pbyDstBuffer = NULL;
        
        option.InVPicture.Width = option.InVPicture.Height = option.InVPicture.Stride = dwFovSize;
        option.InVPicture.Format = FE_PIXELFORMAT_YUV420P;
        
        dwSrcBufferSize = option.InVPicture.Stride * option.InVPicture.Height * 3 / 2;
        
        if (NULL == (pbySrcBuffer = (BYTE *)malloc(dwSrcBufferSize)))
        {
            goto RELEASE_MEMORY;
        }
        
        memset(pbySrcBuffer, 0, dwSrcBufferSize);
        option.InVPicture.Buffer = pbySrcBuffer;
        
        fread(option.InVPicture.Buffer, dwSrcBufferSize, 1, pfImage);
        fclose(pfImage);
        pfImage = NULL;
        
        // set ouput image information
        option.OutVPicture.Width = option.OutVPicture.Stride = WINDOW_WIDTH;
        option.OutVPicture.Height = WINDOW_HEIGHT;
        option.OutVPicture.Format = FE_PIXELFORMAT_YUV420P;
        dwDstBufferSize = option.OutVPicture.Stride * option.OutVPicture.Height * 3 / 2;
        
        if (NULL == (pbyDstBuffer = (BYTE *)malloc(dwDstBufferSize)))
        {
            goto RELEASE_MEMORY;
        }
        
        option.OutVPicture.Buffer = pbyDstBuffer;
    }

    // Set FOV information
    option.FOVCenter.X = option.InVPicture.Width >> 1;
    option.FOVCenter.Y = option.InVPicture.Height >> 1;
    option.FOVRadius = option.InVPicture.Height >> 1;
    
    // Set display output region
    option.OutRoi.Left = 0;
    option.OutRoi.Top = 0;
    option.OutRoi.Right = WINDOW_WIDTH;
    option.OutRoi.Bottom = WINDOW_HEIGHT;
    
    // Set mount type
    switch (mountType)
    {
        case 0:
            option.MountType = FE_MOUNT_WALL;
            break;
        case 1:
            option.MountType = FE_MOUNT_CEILING;
            break;
        case 2:
            option.MountType = FE_MOUNT_FLOOR;
            break;
        default:
            break;
    }
    
    // Set dewarp type
    option.DewarpType = FE_DEWARP_RECTILINEAR;
    
    // Set flags
    option.Flags = (FE_OPTION_INIMAGEHEADER | FE_OPTION_INIMAGEBUFFER | FE_OPTION_OUTIMAGEHEADER | FE_OPTION_OUTIMAGEBUFFER |
                    FE_OPTION_FOVCENTER | FE_OPTION_FOVRADIUS | FE_OPTION_OUTROI | FE_OPTION_MOUNTTYPE | FE_OPTION_DEWARPTYPE | FE_OPTION_SCALE);
    
    
    // 3. Update options in fisheyesw library and paint
    UpdateOutVPicture(hFECtx, &option, pTexture, pRenderer);
    
    // 4. Wait keyboard and mouse event to perform ePTZ control
    EventLoop(hFECtx, &option, pTexture, pRenderer);
    
RELEASE_MEMORY:
    
    // release buffer memory by user
    if (option.OutVPicture.Buffer)
    {
        free(option.OutVPicture.Buffer);
        option.OutVPicture.Buffer = NULL;
    }
    if (hFECtx)
    {
        // Release Fisheye library.
        Fisheye_Release(&hFECtx);
        hFECtx = NULL;
    }
    
    // release SDL resource
    if (pBitmap)
    {
        SDL_FreeSurface(pBitmap);
        pBitmap = NULL;
    }
    if (pTexture)
    {
        SDL_DestroyTexture(pTexture);
        pTexture = NULL;
    }
    if (pRenderer)
    {
        SDL_DestroyRenderer(pRenderer);
        pRenderer = NULL;
    }
    if (pScreen)
    {
        SDL_DestroyWindow(pScreen);
        pScreen = NULL;
    }
  
    SDL_Quit();
    
    return 0;
}

void EventLoop(HANDLE hObject, FEOPTION *ptOption, SDL_Texture* pTexture, SDL_Renderer *pRenderer)
{
    int bExit = 0;
    SDL_Event evt;
    SDL_MouseButtonEvent beginPos = { 0 };
    int bClick = 0;
    
    while (!bExit)
    {
        SDL_WaitEvent(&evt);
        
        // clear flags
        ptOption->Flags = 0;
        
        switch (evt.type)
        {
            case SDL_KEYDOWN:
                switch (evt.key.keysym.sym)
            {
                case SDLK_UP:
                    ptOption->Pitch += 1;
                    ptOption->Flags |= FE_OPTION_PITCH;
                    break;
                case SDLK_DOWN:
                    ptOption->Pitch -= 1;
                    ptOption->Flags |= FE_OPTION_PITCH;
                    break;
                case SDLK_LEFT:
                    if (FE_MOUNT_WALL == ptOption->MountType)
                    {
                        ptOption->Yaw -= 1;
                        ptOption->Flags |= FE_OPTION_YAW;
                    }
                    else
                    {
                        ptOption->Roll -= 1;
                        ptOption->Flags |= FE_OPTION_ROLL;
                    }
                    break;
                case SDLK_RIGHT:
                    if (FE_MOUNT_WALL == ptOption->MountType)
                    {
                        ptOption->Yaw += 1;
                        ptOption->Flags |= FE_OPTION_YAW;
                    }
                    else
                    {
                        ptOption->Roll += 1;
                        ptOption->Flags |= FE_OPTION_ROLL;
                    }
                    break;
                case SDLK_KP_PLUS:
                    ptOption->Scale += 1;
                    ptOption->Flags |= FE_OPTION_SCALE;
                    break;
                case SDLK_KP_MINUS:
                    ptOption->Scale -= 1;
                    ptOption->Flags |= FE_OPTION_SCALE;
                default:
                    break;
            }
                // set option and repaint
                UpdateOutVPicture(hObject, ptOption, pTexture, pRenderer);
                break;
            case SDL_MOUSEBUTTONDOWN:
                beginPos = evt.button;  // setup the begin position when mouse click
                bClick = 1;             // enable drag image
                break;
            case SDL_MOUSEBUTTONUP:
                bClick = 0;             // disable drag image
                break;
            case SDL_MOUSEMOTION:
                if (bClick)
                {
                    // calculate the difference between begin position and current position
                    SDL_MouseButtonEvent movePos = evt.button;
                    int iDistance_X = (movePos.x - beginPos.x);
                    int iDistance_Y = (movePos.y - beginPos.y);
                    
                    beginPos = movePos;  // update begin position to current position
                    
                    if (FE_DEWARP_RECTILINEAR == ptOption->DewarpType)
                    {
                        // move right/left
                        if (FE_MOUNT_WALL == ptOption->MountType)
                        {
                            ptOption->Yaw -= iDistance_X;
                            ptOption->Flags = FE_OPTION_YAW;
                        }
                        else
                        {
                            ptOption->Roll -= iDistance_X;
                            ptOption->Flags = FE_OPTION_ROLL;
                        }
                        
                        // move up/down
                        ptOption->Pitch += iDistance_Y;
                        ptOption->Flags |= FE_OPTION_PITCH;
                    }
                    
                    // set option and repaint
                    UpdateOutVPicture(hObject, ptOption, pTexture, pRenderer);
                }
                break;
            case SDL_MOUSEWHEEL:
                ptOption->Scale += evt.wheel.y;
                ptOption->Flags = FE_OPTION_SCALE;
                
                // set option and repaint
                UpdateOutVPicture(hObject, ptOption, pTexture, pRenderer);
                break;
            case SDL_QUIT:
                bExit = 1;
                break;
            default:
                break;
        }
    }
}

SCODE UpdateOutVPicture(HANDLE hObject, FEOPTION *ptOption, SDL_Texture* pTexture, SDL_Renderer *pRenderer)
{
    // set fisheye option, use Flags to control which field of FEOPTION needs to be updated.
    SCODE scRet = S_OK;
    scRet = Fisheye_SetOption(hObject, ptOption);
    if (S_OK != scRet)
    {
        return scRet;
    }
    
    // update dewarp resust to OutVPicture.Buffer
    scRet = Fisheye_OneFrame(hObject);
    if (S_OK != scRet)
    {
        printf("Fisheye_OneFrame failed : %x \n", scRet);
    }
    
    // SDL display
    SDL_UpdateTexture(pTexture, NULL, ptOption->OutVPicture.Buffer, ptOption->OutVPicture.Stride);
    SDL_RenderClear(pRenderer);
    SDL_RenderCopy(pRenderer, pTexture, NULL, NULL);
    SDL_RenderPresent(pRenderer);
    
    return S_OK;
}
